@@ -9,13 +9,14 @@ from warnings import warn
 
 import numpy as np
 
-from scipy._lib.six import xrange, zip as izip
+from scipy._lib.six import zip as izip
 
 from ._sparsetools import coo_tocsr, coo_todense, coo_matvec
-from .base import isspmatrix, SparseEfficiencyWarning
+from .base import isspmatrix, SparseEfficiencyWarning, spmatrix
 from .data import _data_matrix, _minmax_mixin
 from .sputils import (upcast, upcast_char, to_native, isshape, getdtype,
-        isintlike, get_index_dtype, downcast_intp_index)
+                      get_index_dtype, downcast_intp_index, check_shape,
+                      check_reshape_kwargs)
 
 
 class coo_matrix(_data_matrix, _minmax_mixin):
@@ -87,12 +88,15 @@ class coo_matrix(_data_matrix, _minmax_mixin):
 
     Examples
     --------
+    
+    >>> # Constructing an empty matrix
     >>> from scipy.sparse import coo_matrix
     >>> coo_matrix((3, 4), dtype=np.int8).toarray()
     array([[0, 0, 0, 0],
            [0, 0, 0, 0],
            [0, 0, 0, 0]], dtype=int8)
 
+    >>> # Constructing a matrix using ijv format
     >>> row  = np.array([0, 3, 1, 0])
     >>> col  = np.array([0, 3, 1, 2])
     >>> data = np.array([4, 5, 7, 9])
@@ -102,24 +106,30 @@ class coo_matrix(_data_matrix, _minmax_mixin):
            [0, 0, 0, 0],
            [0, 0, 0, 5]])
 
-    >>> # example with duplicates
+    >>> # Constructing a matrix with duplicate indices
     >>> row  = np.array([0, 0, 1, 3, 1, 0, 0])
     >>> col  = np.array([0, 2, 1, 3, 1, 0, 0])
     >>> data = np.array([1, 1, 1, 1, 1, 1, 1])
-    >>> coo_matrix((data, (row, col)), shape=(4, 4)).toarray()
+    >>> coo = coo_matrix((data, (row, col)), shape=(4, 4))
+    >>> # Duplicate indices are maintained until implicitly or explicitly summed
+    >>> np.max(coo.data)
+    1
+    >>> coo.toarray()
     array([[3, 0, 1, 0],
            [0, 2, 0, 0],
            [0, 0, 0, 0],
            [0, 0, 0, 1]])
 
     """
+    format = 'coo'
+
     def __init__(self, arg1, shape=None, dtype=None, copy=False):
         _data_matrix.__init__(self)
 
         if isinstance(arg1, tuple):
             if isshape(arg1):
                 M, N = arg1
-                self.shape = (M,N)
+                self._shape = check_shape((M, N))
                 idx_dtype = get_index_dtype(maxval=max(M, N))
                 self.row = np.array([], dtype=idx_dtype)
                 self.col = np.array([], dtype=idx_dtype)
@@ -137,11 +147,11 @@ class coo_matrix(_data_matrix, _minmax_mixin):
                                          'sized index arrays')
                     M = np.max(row) + 1
                     N = np.max(col) + 1
-                    self.shape = (M, N)
+                    self._shape = check_shape((M, N))
                 else:
                     # Use 2 steps to ensure shape has length 2.
                     M, N = shape
-                    self.shape = (M, N)
+                    self._shape = check_shape((M, N))
 
                 idx_dtype = get_index_dtype(maxval=max(self.shape))
                 self.row = np.array(row, copy=copy, dtype=idx_dtype)
@@ -155,13 +165,13 @@ class coo_matrix(_data_matrix, _minmax_mixin):
                     self.row = arg1.row.copy()
                     self.col = arg1.col.copy()
                     self.data = arg1.data.copy()
-                    self.shape = arg1.shape
+                    self._shape = check_shape(arg1.shape)
                 else:
                     coo = arg1.tocoo()
                     self.row = coo.row
                     self.col = coo.col
                     self.data = coo.data
-                    self.shape = coo.shape
+                    self._shape = check_shape(coo.shape)
                 self.has_canonical_format = False
             else:
                 #dense argument
@@ -170,26 +180,52 @@ class coo_matrix(_data_matrix, _minmax_mixin):
                 if M.ndim != 2:
                     raise TypeError('expected dimension <= 2 array or matrix')
                 else:
-                    self.shape = M.shape
+                    self._shape = check_shape(M.shape)
 
                 self.row, self.col = M.nonzero()
                 self.data = M[self.row, self.col]
                 self.has_canonical_format = True
 
         if dtype is not None:
-            self.data = self.data.astype(dtype)
+            self.data = self.data.astype(dtype, copy=False)
 
         self._check()
 
-    def getnnz(self, axis=None):
-        """Get the count of explicitly-stored values (nonzeros)
+    def reshape(self, *args, **kwargs):
+        shape = check_shape(args, self.shape)
+        order, copy = check_reshape_kwargs(kwargs)
 
-        Parameters
-        ----------
-        axis : None, 0, or 1
-            Select between the number of values across the whole matrix, in
-            each column, or in each row.
-        """
+        # Return early if reshape is not required
+        if shape == self.shape:
+            if copy:
+                return self.copy()
+            else:
+                return self
+
+        nrows, ncols = self.shape
+
+        if order == 'C':
+            flat_indices = ncols * self.row + self.col
+            new_row, new_col = divmod(flat_indices, shape[1])
+        elif order == 'F':
+            flat_indices = self.row + nrows * self.col
+            new_col, new_row = divmod(flat_indices, shape[0])
+        else:
+            raise ValueError("'order' must be 'C' or 'F'")
+
+        # Handle copy here rather than passing on to the constructor so that no
+        # copy will be made of new_row and new_col regardless
+        if copy:
+            new_data = self.data.copy()
+        else:
+            new_data = self.data
+
+        return coo_matrix((new_data, (new_row, new_col)),
+                          shape=shape, copy=False)
+
+    reshape.__doc__ = spmatrix.reshape.__doc__
+
+    def getnnz(self, axis=None):
         if axis is None:
             nnz = len(self.data)
             if nnz != len(self.row) or nnz != len(self.col):
@@ -212,11 +248,11 @@ class coo_matrix(_data_matrix, _minmax_mixin):
                                minlength=self.shape[0])
         else:
             raise ValueError('axis out of bounds')
-    nnz = property(fget=getnnz)
+
+    getnnz.__doc__ = spmatrix.getnnz.__doc__
 
     def _check(self):
         """ Checks data structure for consistency """
-        nnz = self.nnz
 
         # index arrays should have integer data types
         if self.row.dtype.kind != 'i':
@@ -231,7 +267,7 @@ class coo_matrix(_data_matrix, _minmax_mixin):
         self.col = np.asarray(self.col, dtype=idx_dtype)
         self.data = to_native(self.data)
 
-        if nnz > 0:
+        if self.nnz > 0:
             if self.row.max() >= self.shape[0]:
                 raise ValueError('row index exceeds matrix dimensions')
             if self.col.max() >= self.shape[1]:
@@ -241,9 +277,33 @@ class coo_matrix(_data_matrix, _minmax_mixin):
             if self.col.min() < 0:
                 raise ValueError('negative column index found')
 
-    def transpose(self, copy=False):
-        M,N = self.shape
-        return coo_matrix((self.data, (self.col, self.row)), shape=(N,M), copy=copy)
+    def transpose(self, axes=None, copy=False):
+        if axes is not None:
+            raise ValueError(("Sparse matrices do not support "
+                              "an 'axes' parameter because swapping "
+                              "dimensions is the only logical permutation."))
+
+        M, N = self.shape
+        return coo_matrix((self.data, (self.col, self.row)),
+                          shape=(N, M), copy=copy)
+
+    transpose.__doc__ = spmatrix.transpose.__doc__
+
+    def resize(self, *shape):
+        shape = check_shape(shape)
+        new_M, new_N = shape
+        M, N = self.shape
+
+        if new_M < M or new_N < N:
+            mask = np.logical_and(self.row < new_M, self.col < new_N)
+            if not mask.all():
+                self.row = self.row[mask]
+                self.col = self.col[mask]
+                self.data = self.data[mask]
+
+        self._shape = shape
+
+    resize.__doc__ = spmatrix.resize.__doc__
 
     def toarray(self, order=None, out=None):
         """See the docstring for `spmatrix.toarray`."""
@@ -256,8 +316,8 @@ class coo_matrix(_data_matrix, _minmax_mixin):
                     B.ravel('A'), fortran)
         return B
 
-    def tocsc(self):
-        """Return a copy of this matrix in Compressed Sparse Column format
+    def tocsc(self, copy=False):
+        """Convert this matrix to Compressed Sparse Column format
 
         Duplicate entries will be summed together.
 
@@ -283,23 +343,23 @@ class coo_matrix(_data_matrix, _minmax_mixin):
             M,N = self.shape
             idx_dtype = get_index_dtype((self.col, self.row),
                                         maxval=max(self.nnz, M))
-            indptr = np.empty(N + 1, dtype=idx_dtype)
-            indices = np.empty(self.nnz, dtype=idx_dtype)
-            data = np.empty(self.nnz, dtype=upcast(self.dtype))
+            row = self.row.astype(idx_dtype, copy=False)
+            col = self.col.astype(idx_dtype, copy=False)
 
-            coo_tocsr(N, M, self.nnz,
-                      self.col.astype(idx_dtype),
-                      self.row.astype(idx_dtype),
-                      self.data,
+            indptr = np.empty(N + 1, dtype=idx_dtype)
+            indices = np.empty_like(row, dtype=idx_dtype)
+            data = np.empty_like(self.data, dtype=upcast(self.dtype))
+
+            coo_tocsr(N, M, self.nnz, col, row, self.data,
                       indptr, indices, data)
 
-            A = csc_matrix((data, indices, indptr), shape=self.shape)
-            A.sum_duplicates()
+            x = csc_matrix((data, indices, indptr), shape=self.shape)
+            if not self.has_canonical_format:
+                x.sum_duplicates()
+            return x
 
-            return A
-
-    def tocsr(self):
-        """Return a copy of this matrix in Compressed Sparse Row format
+    def tocsr(self, copy=False):
+        """Convert this matrix to Compressed Sparse Row format
 
         Duplicate entries will be summed together.
 
@@ -325,22 +385,20 @@ class coo_matrix(_data_matrix, _minmax_mixin):
             M,N = self.shape
             idx_dtype = get_index_dtype((self.row, self.col),
                                         maxval=max(self.nnz, N))
+            row = self.row.astype(idx_dtype, copy=False)
+            col = self.col.astype(idx_dtype, copy=False)
+
             indptr = np.empty(M + 1, dtype=idx_dtype)
-            indices = np.empty(self.nnz, dtype=idx_dtype)
-            data = np.empty(self.nnz, dtype=upcast(self.dtype))
+            indices = np.empty_like(col, dtype=idx_dtype)
+            data = np.empty_like(self.data, dtype=upcast(self.dtype))
 
-            coo_tocsr(M, N, self.nnz,
-                      self.row.astype(idx_dtype),
-                      self.col.astype(idx_dtype),
-                      self.data,
-                      indptr,
-                      indices,
-                      data)
+            coo_tocsr(M, N, self.nnz, row, col, self.data,
+                      indptr, indices, data)
 
-            A = csr_matrix((data, indices, indptr), shape=self.shape)
-            A.sum_duplicates()
-
-            return A
+            x = csr_matrix((data, indices, indptr), shape=self.shape)
+            if not self.has_canonical_format:
+                x.sum_duplicates()
+            return x
 
     def tocoo(self, copy=False):
         if copy:
@@ -348,9 +406,12 @@ class coo_matrix(_data_matrix, _minmax_mixin):
         else:
             return self
 
-    def todia(self):
+    tocoo.__doc__ = spmatrix.tocoo.__doc__
+
+    def todia(self, copy=False):
         from .dia import dia_matrix
 
+        self.sum_duplicates()
         ks = self.col - self.row  # the diagonal for each nonzero
         diags, diag_idx = np.unique(ks, return_inverse=True)
 
@@ -368,26 +429,38 @@ class coo_matrix(_data_matrix, _minmax_mixin):
 
         return dia_matrix((data,diags), shape=self.shape)
 
-    def todok(self):
+    todia.__doc__ = spmatrix.todia.__doc__
+
+    def todok(self, copy=False):
         from .dok import dok_matrix
 
         self.sum_duplicates()
         dok = dok_matrix((self.shape), dtype=self.dtype)
-        dok.update(izip(izip(self.row,self.col),self.data))
+        dok._update(izip(izip(self.row,self.col),self.data))
 
         return dok
 
-    def diagonal(self):
-        # Could be rewritten without the python loop.
-        # Data entries at the same (row, col) are summed.
-        n = min(self.shape)
-        ndata = self.data.shape[0]
-        d = np.zeros(n, dtype=self.dtype)
-        for i in xrange(ndata):
-            r = self.row[i]
-            if r == self.col[i]:
-                d[r] += self.data[i]
-        return d
+    todok.__doc__ = spmatrix.todok.__doc__
+
+    def diagonal(self, k=0):
+        rows, cols = self.shape
+        if k <= -rows or k >= cols:
+            raise ValueError("k exceeds matrix dimensions")
+        diag = np.zeros(min(rows + min(k, 0), cols - max(k, 0)),
+                        dtype=self.dtype)
+        diag_mask = (self.row + k) == self.col
+
+        if self.has_canonical_format:
+            row = self.row[diag_mask]
+            data = self.data[diag_mask]
+        else:
+            row, _, data = self._sum_duplicates(self.row[diag_mask],
+                                                self.col[diag_mask],
+                                                self.data[diag_mask])
+        diag[row + min(k, 0)] = data
+
+        return diag
+
     diagonal.__doc__ = _data_matrix.diagonal.__doc__
 
     def _setdiag(self, values, k):
@@ -444,24 +517,53 @@ class coo_matrix(_data_matrix, _minmax_mixin):
 
         This is an *in place* operation
         """
-        if self.has_canonical_format or len(self.data) == 0:
+        if self.has_canonical_format:
             return
-        order = np.lexsort((self.row,self.col))
-        self.row = self.row[order]
-        self.col = self.col[order]
-        self.data = self.data[order]
-        unique_mask = ((self.row[1:] != self.row[:-1]) |
-                       (self.col[1:] != self.col[:-1]))
-        unique_mask = np.append(True, unique_mask)
-        self.row = self.row[unique_mask]
-        self.col = self.col[unique_mask]
-        unique_inds, = np.nonzero(unique_mask)
-        self.data = np.add.reduceat(self.data, unique_inds, dtype=self.dtype)
+        summed = self._sum_duplicates(self.row, self.col, self.data)
+        self.row, self.col, self.data = summed
         self.has_canonical_format = True
 
-    ###########################
-    # Multiplication handlers #
-    ###########################
+    def _sum_duplicates(self, row, col, data):
+        # Assumes (data, row, col) not in canonical format.
+        if len(data) == 0:
+            return row, col, data
+        order = np.lexsort((row, col))
+        row = row[order]
+        col = col[order]
+        data = data[order]
+        unique_mask = ((row[1:] != row[:-1]) |
+                       (col[1:] != col[:-1]))
+        unique_mask = np.append(True, unique_mask)
+        row = row[unique_mask]
+        col = col[unique_mask]
+        unique_inds, = np.nonzero(unique_mask)
+        data = np.add.reduceat(data, unique_inds, dtype=self.dtype)
+        return row, col, data
+
+    def eliminate_zeros(self):
+        """Remove zero entries from the matrix
+
+        This is an *in place* operation
+        """
+        mask = self.data != 0
+        self.data = self.data[mask]
+        self.row = self.row[mask]
+        self.col = self.col[mask]
+
+    #######################
+    # Arithmetic handlers #
+    #######################
+
+    def _add_dense(self, other):
+        if other.shape != self.shape:
+            raise ValueError('Incompatible shapes.')
+        dtype = upcast_char(self.dtype.char, other.dtype.char)
+        result = np.array(other, dtype=dtype, copy=True)
+        fortran = int(result.flags.f_contiguous)
+        M, N = self.shape
+        coo_todense(M, N, self.nnz, self.row, self.col, self.data,
+                    result.ravel('A'), fortran)
+        return np.matrix(result, copy=False)
 
     def _mul_vector(self, other):
         #output array
@@ -471,8 +573,34 @@ class coo_matrix(_data_matrix, _minmax_mixin):
         return result
 
     def _mul_multivector(self, other):
-        return np.hstack([self._mul_vector(col).reshape(-1,1) for col in other.T])
+        result = np.zeros((other.shape[1], self.shape[0]),
+                          dtype=upcast_char(self.dtype.char, other.dtype.char))
+        for i, col in enumerate(other.T):
+            coo_matvec(self.nnz, self.row, self.col, self.data, col, result[i])
+        return result.T.view(type=type(other))
 
 
 def isspmatrix_coo(x):
+    """Is x of coo_matrix type?
+
+    Parameters
+    ----------
+    x
+        object to check for being a coo matrix
+
+    Returns
+    -------
+    bool
+        True if x is a coo matrix, False otherwise
+
+    Examples
+    --------
+    >>> from scipy.sparse import coo_matrix, isspmatrix_coo
+    >>> isspmatrix_coo(coo_matrix([[5]]))
+    True
+
+    >>> from scipy.sparse import coo_matrix, csr_matrix, isspmatrix_coo
+    >>> isspmatrix_coo(csr_matrix([[5]]))
+    False
+    """
     return isinstance(x, coo_matrix)
